//
//  Instance.swift
//  MetalRenderer
//
//  Created by Mira Young on 1/21/20.
//  Copyright © 2020 Mira Young. All rights reserved.
//

import Foundation
import MetalKit

class Instance: Model {
    
    var transforms: [Transform]
    var instanceCount: Int
    var instanceBuffer: MTLBuffer
    
    init(name: String, instanceCount: Int = 1) {
        
        self.instanceCount = instanceCount
        
        transforms = [Transform](repeatElement(Transform(), count: instanceCount))
        instanceBuffer = Renderer.device.makeBuffer(length: instanceCount * MemoryLayout<Instances>.stride, options: [])!
        
        super.init(name: name)
    }
    
    override func render(commandEncoder: MTLRenderCommandEncoder, submesh: Submesh) {
        
        let mtkSubmesh = submesh.mtkSubmesh
        
        var pointer = instanceBuffer.contents().bindMemory(to: Instances.self, capacity: instanceCount)
        
        for transform in transforms {
            if mtkSubmesh.name == "Cylinder.001_Cylinder.006_Leaves" {
                var transformCopy = transform
                transformCopy.position.y = 2
                pointer.pointee.modelMatrix = transformCopy.matrix
                pointer = pointer.advanced(by: 1)
                print("🐷 pointer.pointee.modelMatrix: \(pointer.pointee.modelMatrix)")
            } else {
                pointer.pointee.modelMatrix = transform.matrix
                pointer = pointer.advanced(by: 1)
                print("🦊 pointer.pointee.modelMatrix: \(pointer.pointee.modelMatrix)")
            }
        }
        
        commandEncoder.setVertexBuffer(instanceBuffer, offset: 0, index: 20)
        commandEncoder.setRenderPipelineState(submesh.instancedPipelineState)
        
        commandEncoder.drawIndexedPrimitives(type: .triangle,
                                             indexCount: mtkSubmesh.indexCount,
                                             indexType: mtkSubmesh.indexType,
                                             indexBuffer: mtkSubmesh.indexBuffer.buffer,
                                             indexBufferOffset: mtkSubmesh.indexBuffer.offset,
                                             instanceCount: instanceCount)

    }
    
}
