//
//  ViewController.swift
//  MetalRenderer
//
//  Created by Mira Young on 1/15/20.
//  Copyright © 2020 Mira Young. All rights reserved.
//

import Cocoa
import MetalKit

class ViewController: NSViewController {

    @IBOutlet var metalView: MTKView!
    var renderer: Renderer?
    var scene: Scene?
    
    override func viewDidLoad() {
        super.viewDidLoad()

        // Do any additional setup after loading the view.
        renderer = Renderer(view: metalView)
        scene = GameScene(sceneSize: metalView.bounds.size)
        renderer?.scene = scene
        
        metalView.device = Renderer.device
        metalView.delegate = renderer
        metalView.clearColor = MTLClearColorMake(1, 1, 0.8, 1)
        
        let pan = NSPanGestureRecognizer(target: self, action: #selector(handlePan))
        
        view.addGestureRecognizer(pan)
    }
    
    override func scrollWheel(with event: NSEvent) {
        scene?.camera.zoom(delta: Float(event.deltaY))
    }
    
    @objc func handlePan(gesture: NSPanGestureRecognizer) {
        
        let translation = gesture.translation(in: gesture.view)
        let delta = SIMD2<Float>(Float(translation.x), Float(translation.y))
        
        scene?.camera.rotate(delta: delta)
        gesture.setTranslation(.zero, in: gesture.view)
    }

}

